@@ -8,18 +8,18 @@
 #   list object containing hashes representing records in the file.
 #
 # AUTHOR
-#   Andy Wardley   <abw@cre.canon.co.uk>
+#   Andy Wardley   <abw@kfs.org>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-1999 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-1999 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Datafile.pm,v 1.2 2000/03/28 14:17:47 abw Exp $
+# $Id: Datafile.pm,v 2.3 2000/11/14 15:54:58 abw Exp $
 #
 #============================================================================
 
@@ -30,9 +30,9 @@ require 5.004;
 use strict;
 use vars qw( @ISA $VERSION );
 use base qw( Template::Plugin );
+use Template::Plugin;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
-
+$VERSION = sprintf("%d.%02d", q$Revision: 2.3 $ =~ /(\d+)\.(\d+)/);
 
 sub new {
     my ($class, $context, $filename, $params) = @_;
@@ -52,7 +52,10 @@ sub new {
 	|| return $class->fail("$filename: $!");
 
     # first line of file should contain field definitions
-    $line = <FD>;
+    while (! $line || $line =~ /^#/) {
+	$line = <FD>;
+	chomp $line;
+    }
     (@fields = split(/\s*$delim\s*/, $line)) 
 	|| return $class->fail("first line of file must contain field names");
 
@@ -73,11 +76,13 @@ sub new {
 	push(@$self, \%record);
     }
 
+#    return $self;
     bless $self, $class;
 }	
 
 
-sub AUTOLOAD {
+sub as_list {
+    return $_[0];
 }
 
 
@@ -144,11 +149,11 @@ Doesn't permit use of ':' in a field.  Some escaping mechanism is required.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>cre.canon.co.ukE<gt>
+Andy Wardley E<lt>kfs.orgE<gt>
 
 =head1 REVISION
 
-$Revision: 1.2 $
+$Revision: 2.3 $
 
 =head1 COPYRIGHT
 
