@@ -7,18 +7,18 @@
 #   Simple Template Toolkit Plugin which creates formatting functions.
 #
 # AUTHOR
-#   Andy Wardley   <abw@cre.canon.co.uk>
+#   Andy Wardley   <abw@kfs.org>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-1999 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-1999 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Format.pm,v 1.4 2000/03/28 14:17:47 abw Exp $
+# $Id: Format.pm,v 2.2 2000/11/01 12:01:43 abw Exp $
 #
 #============================================================================
 
@@ -29,9 +29,9 @@ require 5.004;
 use strict;
 use vars qw( @ISA $VERSION );
 use base qw( Template::Plugin );
-use CGI;
+use Template::Plugin;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.2 $ =~ /(\d+)\.(\d+)/);
 
 
 sub new {
@@ -73,11 +73,11 @@ a printf()-like format string.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>cre.canon.co.ukE<gt>
+Andy Wardley E<lt>kfs.orgE<gt>
 
 =head1 REVISION
 
-$Revision: 1.4 $
+$Revision: 2.2 $
 
 =head1 COPYRIGHT
 
