@@ -8,7 +8,7 @@
 #   from which row and column permutations can be fetched.
 #
 # AUTHOR
-#   Andy Wardley   <abw@cre.canon.co.uk>
+#   Andy Wardley   <abw@kfs.org>
 #
 # COPYRIGHT
 #   Copyright (C) 2000 Andy Wardley.  All Rights Reserved.
@@ -18,7 +18,7 @@
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Table.pm,v 1.2 2000/03/27 12:33:31 abw Exp $
+# $Id: Table.pm,v 2.2 2000/10/09 11:14:45 abw Exp $
 #
 #============================================================================
 
@@ -28,11 +28,10 @@ require 5.004;
 
 use strict;
 use vars qw( @ISA $VERSION $AUTOLOAD );
+use base qw( Template::Plugin );
 use Template::Plugin;
-use CGI;
 
-@ISA     = qw( Template::Plugin );
-$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.2 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -78,6 +77,7 @@ sub new {
 	if ($size < $rows) {
 	    $rows = $size;   # pad?
 	    $cols = 1;
+	    $coloff = 0;
 	}
 	else {
 	    $coloff = $rows - $overlap;
@@ -90,6 +90,7 @@ sub new {
 	if ($size < $cols) {
 	    $cols = $size;
 	    $rows = 1;
+	    $coloff = 1;
 	}
 	else {
 	    $coloff = int ($size / $cols) 
@@ -403,11 +404,11 @@ remaining items.  These are then available via the usual Table interface.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@cre.canon.co.ukE<gt>
+Andy Wardley E<lt>abw@kfs.orgE<gt>
 
 =head1 REVISION
 
-$Revision: 1.2 $
+$Revision: 2.2 $
 
 =head1 COPYRIGHT
 
