@@ -8,7 +8,7 @@
 #   and adaptable parameters.
 #
 # AUTHOR
-#   Andy Wardley   <abw@cre.canon.co.uk>
+#   Andy Wardley   <abw@kfs.org>
 #
 # COPYRIGHT
 #   Copyright (C) 2000 Andy Wardley.  All Rights Reserved.
@@ -18,7 +18,7 @@
 #
 #----------------------------------------------------------------------------
 #
-# $Id: URL.pm,v 1.1 2000/03/20 08:02:27 abw Exp $
+# $Id: URL.pm,v 2.2 2000/11/14 15:54:58 abw Exp $
 #
 #============================================================================
 
@@ -29,10 +29,9 @@ require 5.004;
 use strict;
 use vars qw( @ISA $VERSION );
 use Template::Plugin;
-use CGI;
 
 @ISA     = qw( Template::Plugin );
-$VERSION = sprintf("%d.%02d", q$Revision: 1.1 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.2 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -50,7 +49,7 @@ sub new {
 	my $newbase = shift unless ref $_[0] eq 'HASH';
 	my $newargs = shift || { };
 	my $combo   = { %$args, %$newargs };
-	my $urlargs = join('&', 
+	my $urlargs = join('&amp;', 
 			   map  { "$_=" . escape($combo->{ $_ }) }
 			   grep { defined $combo->{ $_ } }
 			   keys %$combo);
@@ -91,19 +90,19 @@ Template::Plugin::URL - constructs query URL's with parameters
     [% USE url('/cgi-bin/foo.pl') %]
 
     [% url(debug = 1, id = 123) %]
-       # ==> /cgi/bin/foo.pl?debug=1&id=123
+       # ==> /cgi/bin/foo.pl?debug=1&amp;id=123
 
 
     [% USE mycgi = url('/cgi-bin/bar.pl', mode='browse', debug=1) %]
 
     [% mycgi %]
-       # ==> /cgi/bin/bar.pl?mode=browse&debug=1
+       # ==> /cgi/bin/bar.pl?mode=browse&amp;debug=1
 
     [% mycgi(mode='submit') %]
-       # ==> /cgi/bin/bar.pl?mode=submit&debug=1
+       # ==> /cgi/bin/bar.pl?mode=submit&amp;debug=1
 
     [% mycgi(debug='d2 p0', id='D4-2k[4]') %]
-       # ==> /cgi-bin/bar.pl?mode=browse&debug=d2%20p0&id=D4-2k%5B4%5D
+       # ==> /cgi-bin/bar.pl?mode=browse&amp;debug=d2%20p0&amp;id=D4-2k%5B4%5D
 
 
 =head1 DESCRIPTION
@@ -128,7 +127,7 @@ For the above three examples, these will produce the following outputs:
 
     http://www.somewhere.com/cgi-bin/foo.pl
     /cgi-bin/bar.pl?mode=browse
-    /cgi-bin/baz.pl?mode=browse&debug=1
+    /cgi-bin/baz.pl?mode=browse&amp;debug=1
 
 Additional parameters may be also be specified:
 
@@ -136,9 +135,9 @@ Additional parameters may be also be specified:
 
 Which, for the same three examples, produces:
 
-    http://www.somewhere.com/cgi-bin/foo.pl?mode=submit&id=wiz
-    /cgi-bin/bar.pl?mode=submit&id=wiz
-    /cgi-bin/baz.pl?mode=browse&debug=1&id=wiz
+    http://www.somewhere.com/cgi-bin/foo.pl?mode=submit&amp;id=wiz
+    /cgi-bin/bar.pl?mode=browse&amp;id=wiz
+    /cgi-bin/baz.pl?mode=browse&amp;debug=1&amp;id=wiz
 
 A new base URL may also be specified as the first option:
 
@@ -147,8 +146,8 @@ A new base URL may also be specified as the first option:
 producing
 
     /cgi-bin/waz.pl?test=1
-    /cgi-bin/waz.pl?mode=browse&test=1
-    /cgi-bin/waz.pl?mode=browse&debug=1&test=1
+    /cgi-bin/waz.pl?mode=browse&amp;test=1
+    /cgi-bin/waz.pl?mode=browse&amp;debug=1&amp;test=1
 
 
 The ordering of the parameters is non-deterministic due to fact that 
@@ -171,14 +170,20 @@ as per regular Template Toolkit syntax.
 
     [% mycgi(debug=1) %]
 
+Note that in the following line, additional parameters are seperated
+by '&amp;', while common usage on the Web is to just use '&'. '&amp;'
+is actually the Right Way to do it. See this URL for more information:
+http://ppewww.ph.gla.ac.uk/~flavell/www/formgetbyurl.html
+
+    /cgi-bin/waz.pl?mode=browse&amp;debug=1&amp;test=1
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@cre.canon.co.ukE<gt>
+Andy Wardley E<lt>abw@kfs.orgE<gt>
 
 =head1 REVISION
 
-$Revision: 1.1 $
+$Revision: 2.2 $
 
 =head1 COPYRIGHT
 
